# Generated by Django 4.2.4 on 2023-08-28 21:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_tweets', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='mediafile',
            name='upload',
        ),
    ]



from django.conf import settings

import tweepy

from .decorators import check_settings


@check_settings
def get_v2_client():
    return tweepy.Client(
        bearer_token=settings.TWITTER_BEARER_TOKEN,
        consumer_key=settings.TWITTER_API_KEY,
        consumer_secret=settings.TWITTER_API_KEY_SECRET,
        access_token=settings.TWITTER_ACCESS_TOKEN,
        access_token_secret=settings.TWITTER_ACCESS_TOKEN_SECRET,
        return_type=tweepy.Response,
    )


@check_settings
def get_v1dot1_api():
    auth = tweepy.OAuthHandler(
        settings.TWITTER_API_KEY,
        settings.TWITTER_API_KEY_SECRET,
    )
    auth.set_access_token(
        settings.TWITTER_ACCESS_TOKEN,
        settings.TWITTER_ACCESS_TOKEN_SECRET,
    )
    return tweepy.API(auth, wait_on_rate_limit=True)



import tempfile

from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.utils.functional import cached_property

from django.conf import settings

from .clients import get_v2_client
from .clients import get_v1dot1_api


def _upload_path(_, filename):
    now = timezone.now()
    return f"django-tweets/{now.year}/{now.month}/{now.day}/{filename}"


class MediaFile(models.Model):
    title = models.CharField(
        _("Title"),
        max_length=128,
        blank=True,
    )
    file = models.FileField(
        _("File"),
        upload_to=_upload_path,
    )
    created_at = models.DateTimeField(
        _("Created at"),
        editable=False,
        blank=True,
        auto_now_add=True,
    )
    expires_at = models.DateTimeField(
        _("Expires at"),
        editable=True,
        blank=True,
        null=True,
    )
    media_id_string = models.CharField(
        max_length=32,
        editable=False,
        blank=True,
        null=True,
    )
    response = models.TextField(
        _("Tweepy Response"),
        editable=True,
        blank=True,
        null=True,
    )

    def upload_file(self):
        # use tempfile to upload the file to the Twitter API.
        # Why tempfile? because not allways media files are not stored locally
        with tempfile.NamedTemporaryFile(suffix="." + self.file_extension) as f:
            f.write(self.file.read())
            f.seek(0)  # https://github.com/tweepy/tweepy/issues/1667
            response = get_v1dot1_api().chunked_upload(f.name)
        # save values into the db
        self.media_id_string = response.media_id_string
        self.response = str(response)
        self.expires_at = timezone.now() + timezone.timedelta(
            seconds=response.expires_after_secs
        )
        self.save()
        return self

    @cached_property
    def file_extension(self):
        return self.file.name.split(".")[1]

    def save(self, *args, **kwargs):
        # generate title from file name if not provided
        if self.title == "" or self.title is None:
            self.title = self.file.name
        super(MediaFile, self).save(*args, **kwargs)

    def __str__(self) -> str:
        return self.title


class Tweet(models.Model):
    text = models.TextField(max_length=4096)
    id_string = models.CharField(max_length=32, editable=False, blank=True)
    edit_history_tweet_ids = models.TextField(
        max_length=512, editable=False, blank=True
    )
    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    files = models.ManyToManyField(MediaFile, blank=True)
    response = models.TextField(
        _("Tweepy Response"),
        editable=True,
        blank=True,
        null=True,
    )

    def delete(self, *args, **kwargs):
        if getattr(settings, "DJANGO_TWEETS_SYNC_DELETE", True):
            get_v2_client().delete_tweet(self.id_string)
        super(Tweet, self).delete(*args, **kwargs)

    def get_media_ids(self):
        if self.files.all().count() > 0:
            return [f.media_id_string for f in self.files.all()]

    def publish(self):
        # requesting
        response = get_v2_client().create_tweet(
            text=self.text,
            media_ids=self.get_media_ids(),
        )
        # updating the instance from the response
        self.id_string = response.data.get("id")
        self.text = response.data.get("text")
        self.edit_history_tweet_ids = ", ".join(
            response.data.get("edit_history_tweet_ids")
        )
        self.save()
        return self

    def __str__(self) -> str:
        return "%s %s" % (self.id_string, self.text)


class TweetPublication(models.Model):
    """
    A Model to handle Tweet publication in Django Admin.
    """

    tweet = models.OneToOneField(Tweet, on_delete=models.CASCADE)
    publish = models.BooleanField(default=False)

    def save(self, *args, **kwargs):
        if self.publish:
            self.tweet.publish()
        super(TweetPublication, self).save(*args, **kwargs)


class MediaFileUpload(models.Model):
    """
    A Model to handle MediaFile upload in Django Admin.
    """

    mediafile = models.OneToOneField(MediaFile, on_delete=models.CASCADE)
    upload = models.BooleanField(default=False)

    def save(self, *args, **kwargs):
        if self.publish:
            self.mediafile.upload_file()
        super(MediaFileUpload, self).save(*args, **kwargs)



# Generated by Django 4.2.4 on 2023-08-28 21:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_tweets', '0002_remove_mediafile_upload'),
    ]

    operations = [
        migrations.CreateModel(
            name='MediaFileUpload',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('upload', models.BooleanField(default=False)),
                ('mediafile', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='django_tweets.mediafile')),
            ],
        ),
    ]



from django.conf import settings

from .exceptions import SettingError


def check_settings(func):
    settings_to_check = (
        "TWITTER_BEARER_TOKEN",
        "TWITTER_API_KEY",
        "TWITTER_API_KEY_SECRET",
        "TWITTER_ACCESS_TOKEN",
        "TWITTER_ACCESS_TOKEN_SECRET",
    )

    def wrapper(*args, **kwargs):
        for setting in settings_to_check:
            if getattr(settings, setting, None) is None:
                raise SettingError("Please provide a setting value to %s" % setting)
        return func(*args, **kwargs)

    return wrapper



# Generated by Django 4.2.4 on 2023-08-28 15:41

from django.db import migrations, models
import django.db.models.deletion
import django_tweets.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MediaFile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=128, verbose_name='Title')),
                ('file', models.FileField(upload_to=django_tweets.models._upload_path, verbose_name='File')),
                ('upload', models.BooleanField(default=True, verbose_name='Upload to Twitter')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('expires_at', models.DateTimeField(blank=True, null=True, verbose_name='Expires at')),
                ('media_id_string', models.CharField(blank=True, editable=False, max_length=32, null=True)),
                ('response', models.TextField(blank=True, null=True, verbose_name='Tweepy Response')),
            ],
        ),
        migrations.CreateModel(
            name='Tweet',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField(max_length=4096)),
                ('id_string', models.CharField(blank=True, editable=False, max_length=32)),
                ('edit_history_tweet_ids', models.TextField(blank=True, editable=False, max_length=512)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('response', models.TextField(blank=True, null=True, verbose_name='Tweepy Response')),
                ('files', models.ManyToManyField(blank=True, to='django_tweets.mediafile')),
            ],
        ),
        migrations.CreateModel(
            name='TweetPublication',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('publish', models.BooleanField(default=False)),
                ('tweet', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='django_tweets.tweet')),
            ],
        ),
    ]



from django.contrib import admin

from .models import MediaFile
from .models import MediaFileUpload
from .models import Tweet
from .models import TweetPublication


@admin.register(Tweet)
class TweetAdmin(admin.ModelAdmin):
    """
    ModelAdmin for Tweet

    """

    list_display = ("__str__", "id_string", "created_at")
    list_filter = ("created_at",)
    readonly_fields = ("id_string", "edit_history_tweet_ids", "created_at", "response")
    filter_horizontal = ("files",)


@admin.register(MediaFile)
class MediaFileAdmin(admin.ModelAdmin):
    """
    ModelAdmin for MediaFile
    """

    list_display = ("title", "file", "created_at", "expires_at")
    list_filter = ("expires_at", "expires_at")
    readonly_fields = ("created_at", "expires_at", "media_id_string", "response")


@admin.register(TweetPublication)
class TweetPublicationAdmin(admin.ModelAdmin):
    """
    ModelAdmin for TweetPublication
    """

    list_display = ("tweet", "publish")
    list_filter = ("publish",)


@admin.register(MediaFileUpload)
class MediaFileUploadAdmin(admin.ModelAdmin):
    """
    ModelAdmin for MediaFileUpload
    """

    list_display = ("mediafile", "upload")
    list_filter = ("upload",)



from pathlib import Path

from django.utils import timezone
from django.conf import settings
from django.core.files.base import ContentFile


from django_tweets.models import Tweet
from django_tweets.models import MediaFile
from django_tweets.models import TweetPublication
from django_tweets.clients import get_v2_client

from unittest_parametrize import parametrize
from unittest_parametrize import ParametrizedTestCase


now = timezone.now()
now_tupple_6 = (now.year, now.month, now.day, now.hour, now.minute, now.second)
now_str = "%s-%s-%s %s:%s:%s" % now_tupple_6


class TweetModelTests(ParametrizedTestCase):
    def setUp(self):
        # Setup run before every test method.
        Tweet.objects.all().delete()
        MediaFile.objects.all().delete()
        TweetPublication.objects.all().delete()

    def tearDown(self):
        # Clean up run after every test method.
        Tweet.objects.all().delete()
        MediaFile.objects.all().delete()
        TweetPublication.objects.all().delete()

    def test_create_and_delete_tweet(self):
        text = "%s Testing django-tweets." % now_str
        tweet = Tweet.objects.create(text=text)
        published_tweet = tweet.publish()
        published_tweet.delete()

    @parametrize(
        ("extension", "expected"),
        [
            ("gif", True),
            ("jpg", True),
            # ("mp4", True), # not working
            ("png", True),
        ],
    )
    def test_mediafiles(self, extension: str, expected: bool):
        path = Path(settings.BASE_DIR / "tests" / "samples" / f"sample.{extension}")
        with open(path, "rb") as f:
            f.seek(0)
            contents = f.read()
        mediafile = MediaFile.objects.create(title="nice photo")
        mediafile.file.save(path.name, ContentFile(contents))
        mediafile = mediafile.upload_file()
        tweet = Tweet.objects.create(
            text="%s Testing a tweet with a %s file" % (now_str, extension)
        )
        tweet.files.add(mediafile)
        tweet.publish()
        tweet.delete()
